#include "Python.h"

static int interrupted;

void
PyErr_SetInterrupt(void)
{
	interrupted = 1;
}

extern int PyErr_CheckSignals(void);

static int
checksignals_witharg(void*)
{
	return PyErr_CheckSignals();
}

static int
interrupt(void*, char *msg)
{
	extern void Py_Exit(int);
	static char message[] =
"python: to interrupt a truly hanging Python program, interrupt once more.\n";

	if(strcmp(msg, "interrupt") != 0)
		return 0;

	switch (interrupted++) {
	case 0:
		break;
	case 1:
		write(2, message, strlen(message));
		break;
	case 2:
		interrupted = 0;
		Py_Exit(1);
		break;
	}
	Py_AddPendingCall(checksignals_witharg, NULL);
	return 1;
}

void
PyOS_InitInterrupts(void)
{
	atnotify(interrupt, 1);
}

void
PyOS_FiniInterrupts(void)
{
	atnotify(interrupt, 0);
}

int
PyOS_InterruptOccurred(void)
{
	if (!interrupted)
		return 0;
	interrupted = 0;
	return 1;
}

/* These are probably wrong. */
char *
Py_GetPath(void)
{
    return "/sys/lib/python";
}

char *
Py_GetPrefix(void)
{
	return "/sys/lib/python";
}

char *
Py_GetExecPrefix(void)
{
	return "/sys/lib/python/386";	// BUG
}

char*
Py_GetProgramFullPath(void)
{
	return "/bin/python";
}

time_t
PyOS_GetLastModificationTime(char *path, FILE *fp)
{
	Dir *d;
	time_t t;

	if((d = dirfstat(fileno(fp))) == nil)
		return -1;
	t = d->mtime;
	free(d);
	return t;
}

/*
 * only checked when strtod, strtoul, atof, etc., return zero.
 * we already link against Python-specific strtol, strtoul, so 
 * the only problem is strtod.
 * perhaps we should link against a copy thats set errno to 
 * be nonzero when overflow occurs.
 *
 *	../Objects/complexobject.c
 *		- uses errno but also maintains it.  not to worry.
 *	../Objects/fileobject.c
 *		- just sets it to zero a lot.
 *	../Objects/intobject.c
 *		- expects strtoul and strtol to set errno on overflow
 *	../Parser/myreadline.c
 *		- correctly ignores errno in favor of PyOS_InterruptOccurred
 *	../Python/bltinmodule.c
 *		- expects strtod to set errno on over or underflow.
 *	../Python/compile.c
 *		- expects strtoul, strtol to set errno
 */
int errno;

static char *consname = "/dev/cons";
int
isatty(int fd)
{
	char buf[256];

	if(fd2path(fd, buf, sizeof buf) < 0)
		return 0;

	return strlen(buf)>=strlen(consname)
		&& strcmp(buf+strlen(buf)-strlen(consname), consname) == 0;
}

int
PyOS_vsnprintf(char *buf, size_t nbuf, char *fmt, va_list arg)
{
	return vsnprintf(buf, nbuf, fmt, arg);
}

sighandler_t
signal(int, sighandler_t)
{
	return NULL;
}

#undef gmtime
#undef localtime
#undef asctime
#undef ctime

struct tm*
tm_gmtime(const time_t *t)
{
	static struct tm tm;

	tm = *(struct tm*)gmtime(*t);
	tm.tm_isdst = tm.tm_zone[2]=='S';
	return &tm;
}

struct tm*
tm_localtime(const time_t *t)
{
	static struct tm tm;

	tm = *(struct tm*)localtime(*t);
	tm.tm_isdst = tm.tm_zone[2]=='S';
	return &tm;
}

char*
tm_asctime(const struct tm *tm)
{
	return asctime((Tm*)tm);
}

char*
tm_ctime(const time_t *t)
{
	return ctime(*t);
}

int
gettimeofday(struct timeval *tv)
{
	vlong n;

	n = nsec();
	tv->tv_sec = n/1000000000;
	tv->tv_usec = (n%1000000000)/1000;
	return 0;
}

int
stat(char *name, struct stat *st)
{
	Dir *d;

	if((d = dirstat(name)) == nil)
		return -1;
	st->st_size = d->length;
	st->st_mode = d->mode;
	free(d);
	return 0;
}

int
fstat(int fd, struct stat *st)
{
	Dir *d;

	if((d = dirfstat(fd)) == nil)
		return -1;
	st->st_size = d->length;
	st->st_mode = d->mode;
	free(d);
	return 0;
}

/* Moroo */
lconv lcdummy;

lconv *localeconv(void)
{
	lcdummy.decimal_point=".";

	return &lcdummy;
}
