""" Fake a socket implementation. """

import plan9

AF_INET = 'ipv4'
AF_INET6 = 'ipv6'
AF_UNSPEC = 'unspecified'

AI_CANONNAME = 2
AI_MASK = 7
AI_NUMERICHOST = 4
AI_PASSIVE = 1

EAI_ADDRFAMILY = -9
EAI_AGAIN = -3
EAI_BADFLAGS = -1
EAI_BADHINTS = -12
EAI_FAIL = -4
EAI_FAMILY = -6
EAI_MEMORY = -10
EAI_NODATA = -5
EAI_NONAME = -2
EAI_PROTOCOL = -13
EAI_SERVICE = -8
EAI_SOCKTYPE = -7
EAI_SYSTEM = -11

def _ip(a, b, c, d):
	return (a<<24) | (b<<16) | (c<<8) | d
INADDR_ALLHOSTS_GROUP = _ip(254, 0, 0, 1)
INADDR_ANY = _ip(0, 0, 0, 0)
INADDR_BROADCAST = _ip(255, 255, 255, 255)
INADDR_LOOPBACK = _ip(127, 0, 0, 1)
INADDR_MAX_LOCAL_GROUP = _ip(254, 0, 0, 255)
INADDR_NONE = _ip(255, 255, 255, 255)
INADDR_UNSPEC_GROUP = _ip(254, 0, 0, 0)

IPPORT_RESERVED = 1024
IPPORT_USERRESERVED = 49151

IPPROTO_AH = 51
IPPROTO_DSTOPTS = 60
IPPROTO_EGP = 8
IPPROTO_EON = 80
IPPROTO_ESP = 50
IPPROTO_FRAGMENT = 44
IPPROTO_GGP = 3
IPPROTO_GRE = 47
IPPROTO_HOPOPTS = 0
IPPROTO_ICMP = 1
IPPROTO_ICMPV6 = 58
IPPROTO_IDP = 22
IPPROTO_IGMP = 2
IPPROTO_IP = 0
IPPROTO_IPCOMP = 108
IPPROTO_IPIP = 4
IPPROTO_IPV4 = 4
IPPROTO_IPV6 = 41
IPPROTO_MAX = 256
IPPROTO_MOBILE = 55
IPPROTO_NONE = 59
IPPROTO_PIM = 103
IPPROTO_PUP = 12
IPPROTO_RAW = 255
IPPROTO_ROUTING = 43
IPPROTO_RSVP = 46
IPPROTO_TCP = 6
IPPROTO_TP = 29
IPPROTO_UDP = 17

IPV6_JOIN_GROUP = 12
IPV6_LEAVE_GROUP = 13
IPV6_MULTICAST_HOPS = 10
IPV6_MULTICAST_IF = 9
IPV6_MULTICAST_LOOP = 11
IPV6_UNICAST_HOPS = 4

IP_ADD_MEMBERSHIP = 12
IP_DEFAULT_MULTICAST_LOOP = 1
IP_DEFAULT_MULTICAST_TTL = 1
IP_DROP_MEMBERSHIP = 13
IP_HDRINCL = 2
IP_MAX_MEMBERSHIPS = 20
IP_MULTICAST_IF = 9
IP_MULTICAST_LOOP = 11
IP_MULTICAST_TTL = 10
IP_OPTIONS = 1
IP_RECVDSTADDR = 7
IP_RECVOPTS = 5
IP_RECVRETOPTS = 6
IP_RETOPTS = 8
IP_TOS = 3
IP_TTL = 4

MSG_CTRUNC = 32
MSG_DONTROUTE = 4
MSG_DONTWAIT = 128
MSG_EOR = 8
MSG_OOB = 1
MSG_PEEK = 2
MSG_TRUNC = 16
MSG_WAITALL = 64

NI_DGRAM = 16
NI_MAXHOST = 256
NI_MAXSERV = 32
NI_NAMEREQD = 8
NI_NOFQDN = 4
NI_NUMERICHOST = 1
NI_NUMERICSERV = 2

SOCK_DGRAM = 2
SOCK_RAW = 3
SOCK_RDM = 4
SOCK_SEQPACKET = 5
SOCK_STREAM = 1

SOL_IP = 0
SOL_SOCKET = 65535
SOL_TCP = 6
SOL_UDP = 17

SOMAXCONN = 128

SO_ACCEPTCONN = 2
SO_BROADCAST = 32
SO_DEBUG = 1
SO_DONTROUTE = 16
SO_ERROR = 4103
SO_KEEPALIVE = 8
SO_LINGER = 128
SO_OOBINLINE = 256
SO_RCVBUF = 4098
SO_RCVLOWAT = 4100
SO_RCVTIMEO = 4102
SO_REUSEADDR = 4
SO_REUSEPORT = 512
SO_SNDBUF = 4097
SO_SNDLOWAT = 4099
SO_SNDTIMEO = 4101
SO_TYPE = 4104
SO_USELOOPBACK = 64

TCP_MAXSEG = 2
TCP_NODELAY = 1

_Close = 0
_Bind = 1
_Announce = 2
_Connect = 3

import exceptions
error = exceptions.OSError
del exceptions

def _ipstr(n):
	return '%d.%d.%d.%d' % (n>>24, (n>>16)&0xFF, (n>>8)&0xFF, n&0xFF)

def _mkaddr(proto, host, port):
	if host=='':
		host = '*'
	return '%s!%s!%d' % (proto, host, port)

def _parseaddr(h):
	if h=='::':
		return _ip(0,0,0,0)
	a = h.split('.')
	if len(a) != 4:
		raise error('bad ip address '+h)
	return _ip(int(a[0]), int(a[1]), int(a[2]), int(a[3]))

def _getaddr(file):
	fd = plan9.open(file, plan9.OREAD)
	try:
		s = plan9.read(fd, 100)
		p = s.find('!')
		if p==-1:
			raise error('bad address in '+file+': '+s)
		h, p = s[:p], s[p+1:]
	finally:
		plan9.close(fd)
	return (h, int(p))

class socket:
	def __init__(self, family=AF_INET, type=SOCK_STREAM, proto=0):
		self.family = family
		self.type = type
		self.proto = proto
		if self.family!=AF_INET:
			raise error('unsupported family')
		if self.type!=SOCK_STREAM:
			raise error('unsupported socket type')
		self._proto = 'tcp'
		self._phase = _Close
		self._fd = -1
		self._dir = None
		self._local = None
		self._remote = None
		self._ctl = -1

	def __del__(self):
		self.close()

	def accept(self):
		lfd,ldir = plan9.listen(self._dir)
		try:
			fd = plan9.accept(lfd, ldir)
		finally:
			plan9.close(lfd)
		s = socket(self.family, self.type, self.proto)
		s._fd = fd
		s._dir = ldir
		s._phase = _Connect
		return (s, s.getpeername())

	def bind(self, address):
		self._local = address
		self._phase = _Bind

	def close(self):
		if self._ctl >= 0:
			plan9.close(self._ctl)
			self._ctl = -1
		if self._fd >= 0:
			plan9.close(self._fd)
			self._fd = -1
		self._dir = None
		self._local = None
		self._remote = None
		self._phase = _Close

	def connect(self, address):
		host, port = address
		local = None
		if self._local:
			h, p = self._local
			local = _mkaddr(self._proto, h, p)
		self._fd, self._dir, self._ctl = plan9.dial(_mkaddr(self._proto, host, port), local, 1)
		self._phase = _Connect

	def connect_ex(self, address):
		try:
			self.connect(address)
			return 0
		except:
			return 1	# supposed to be an errno

	def dup(self):
		s = socket(self.family, self.type, self.proto)
		s._fd = self._fd
		if s._fd >= 0:
			s._fd = plan9.dup(s._fd, -1)
		s._ctl = self._ctl
		if s._ctl >= 0:
			s._ctl = plan9.dup(s._ctl, -1)
		s._dir = self._dir
		s._local = self._local
		s._remote = self._remote
		s._phase = self._phase
		return s

	def fileno(self):
		return self._fd #From 9fans posetd by benavento@gmail.com

	def getpeername(self):
		if self._phase==_Connect or self._phase==_Announce:
			return _getaddr(self._dir+'/remote')
		return '::'

	def getsockname(self):
		if self._phase==_Connect or self._phase==_Announce:
			return _getaddr(self._dir+'/local')
		return '::'

	def getsockopt(self, level, option, buffersize=None):
		raise Exception('unknown socket option %d,%d in getsockopt' % (level, option))

	def listen(self, qsize=0):
		# have no use for qsize
		if not self._local:
			raise Exception('need to bind before listening')
		if self._ctl >= 0:
			raise Exception('already listened')
		h, p = self._local
		local = _mkaddr(self._proto, h, p)
		self._ctl, self._dir = plan9.announce(local)

	def makefile(self, mode='rb', bufsize=8192):
		return plan9.fdopen(plan9.dup(self._fd, -1), mode, bufsize)

	def recv(self, buffersize, flags=0):
		if flags:
			raise Exception('unknown flag 0x%x in recv' % (flags,))
		return plan9.read(self._fd, buffersize)

	def recvfrom(self, buffersize, flags=0):
		raise Exception('recvfrom not supported')

	def send(self, data, flags=0):
		if flags:
			raise Exception('unknown flag 0x%x in send' % (flags,))
		return plan9.write(self._fd, data)

	def sendall(self, data, flags=0):
		if flags:
			raise Exception('unknown flag 0x%x in sendall' % (flags,))
		while data:
			m = plan9.write(self._fd, data)
			data = data[m:]

	def sendto(self, data, flags, address=None):
		if address==None:
			address=flags
			flags=0
		raise Exception('sendto not supported')

	def setblocking(self, flag):
		if not flag:
			raise Exception('cannot set non-blocking i/o')

	def setsockopt(self, level, option, value):
		if level==SOL_SOCKET:
			if option==SO_REUSEADDR or option==SO_REUSEPORT:
				return	# ignore option

		raise Exception('cannot set unknown socket option %d,%d to %d' % (level, option, value))

	def shutdown(self, flag):
		if flag==2:
			self.close()
			return
		raise Exception('cannot half-shutdown socket')

def fromfd(fd, family, type, proto=0):
	# could pick up dir from fd2path
	s = socket(family, type, proto)
	s._fd = fd
	s._phase = _Connect
	s._dir = plan9.fd2path(fd)
	p = s._dir.rfind('/')
	s._dir = s._dir[:p]
	return s

#     getaddrinfo(...)
#         socket.getaddrinfo(host, port [, family, socktype, proto, flags])
#                 --> List of (family, socktype, proto, canonname, sockaddr)
#         
#         Resolve host and port into addrinfo struct.
#     

def _query(file, cmd):
	fd = plan9.open(file, plan9.ORDWR)
	l = []
	try:
		plan9.write(fd, cmd)
		plan9.seek(fd, 0, 0)
		while 1:
			s = plan9.read(fd, 1024)
			if not s:
				break
			l.append(s.split())
	except:
		pass
	plan9.close(fd)
	return l

#     gethostbyaddr(...)
#         gethostbyaddr(host) -> (name, aliaslist, addresslist)
#         
#         Return the true host name, a list of aliases, and a list of IP addresses,
#         for a host.  The host argument is a string giving a host name or IP number.

def gethostbyaddr(h):
	def ptrlookup(h):
		for h,t,alias in _query('/net/dns', h+' ptr'):
			p = alias.find('(')
			if p!=-1:
				alias = alias[:p]
			aliaslist.append(alias)

	def iplookup(h):
		for h,t,addr in _query('/net/dns', h+' ip'):
			addresslist.append(addr)

	aliaslist = []
	addresslist = []
	ptrlookup(h)
	iplookup(h)

	if addresslist:
		ptrlookup(addresslist[0])
	elif aliaslist:
		ptrlookup(aliaslist[0])

	if not aliaslist or not addresslist:
		raise error('could not gethost')

	return aliaslist[0], aliaslist[1:], addresslist

def gethostbyname(h):
	for h,t,ip in _query('/net/dns', h):
		if t=='ip':
			return ip
	for line in _query('/net/cs', '!sys='+h+' ip=*'):
		for tuple in line:
			attr,val = tuple.split('=', 1)
			if attr=='ip':
				return val
	raise error('could not translate '+h)

def gethostbyname_ex(host):
	return gethostbyaddr(host)

def gethostname():
	# BUG
	return plan9.sysname()

#     getnameinfo(...)
#         socket.getnameinfo(sockaddr, flags) --> (host, port)
#         
#         Get host and port for a sockaddr.

def getprotobyname(n):
	raise error('no getprotobyname in plan9')

def getservbyname(serv, proto):
	for line in _query('/net/cs', '!'+proto+'='+serv+' port=*'):
		for tuple in line:
			attr,val = tuple.split('=', 1)
			if attr=='port':
				return int(val)

#     htonl(...)
#         htonl(integer) -> integer
#         
#         Convert a 32-bit integer from host to network byte order.
#     
#     htons(...)
#         htons(integer) -> integer
#         
#         Convert a 16-bit integer from host to network byte order.
#     
#     inet_aton(...)
#         inet_aton(string) -> packed 32-bit IP representation
#         
#         Convert an IP address in string format (123.45.67.89) to the 32-bit packed
#         binary format used in low-level network functions.
#     
#     inet_ntoa(...)
#         inet_ntoa(packed_ip) -> ip_address_string
#         
#         Convert an IP address from 32-bit packed binary format to string format
#     
#     ntohl(...)
#         ntohl(integer) -> integer
#         
#         Convert a 32-bit integer from network to host byte order.
#     
#     ntohs(...)
#         ntohs(integer) -> integer
#         
#         Convert a 16-bit integer from network to host byte order.

