"""Network routines for Plan 9."""

import plan9

class _DS:
	def __init__(self, addr, local):
		self.netdir = None
		self.proto = None
		self.rem = None
		self.local = None

		p = addr.find('!')
		if p==-1:
			self.proto = 'net'
			self.rem = addr
		else:
			proto, self.rem = addr[:p], addr[p+1:]
			if proto[0] == '/':
				p = proto.rfind('/')
				self.netdir, proto = proto[:p], proto[p+1:]
			self.proto = proto

def dial(addr, local=None, retcfd=0):
	ds = _DS(addr, local)
	fd, dir, cfd = _dial(ds)
	if not retcfd:
		plan9.close(cfd)
		cfd = -1
	return fd, dir, cfd

def _dial(ds):
	if ds.netdir:
		return _csdial(ds)

	ds.netdir = '/net'
	try:
		return _csdial(ds)
	except:
		pass

	ds.netdir = '/net.alt'
	try:
		return _csdial(ds)
	except Exception, e:
		if str(e).find('translate')==-1 and str(e).find('does not exist')==-1:
			raise

	# induce earlier error again, to get good traceback
	ds.netdir = '/net'
	return _csdial(ds)

def _csdial(ds):
	# open connection server
	try:
		fd = plan9.open(ds.netdir+'/cs', plan9.ORDWR)
	except:
		return _call(ds.netdir+'/'+ds.proto+'/clone', ds.rem, ds)

	# ask connection server to translate
	try:
		plan9.write(fd, ds.proto+'!'+ds.rem)
	except Exception, e:
		plan9.close(fd)
		raise Exception(str(e)+' translating '+ds.proto+'!'+ds.rem)

	# loop through each address from the connection server
	# till we get one that works
	besterr = None
	lasterr = None
	rv = -1
	try:
		plan9.seek(fd, 0, 0)
	except:
		pass
	while 1:
		try:
			s = plan9.read(fd, 1024)
		except:
			s = ''
		if not s:
			break
		p = s.find(' ')
		if p == -1:
			continue
		try:
			rv = _call(s[:p], s[p+1:], ds)
			break
		except Exception, e:
			lasterr = e
			if str(e).find('does not exist') == -1:
				besterr = e
	plan9.close(fd)
	if rv < 0:
		if besterr:
			del lasterr
			raise besterr
		elif lasterr:
			raise lasterr
		else:
			raise Exception('unknown error in csdial')
	return rv

def _call(clone, dest, ds):
	cfd = plan9.open(clone, plan9.ORDWR)
	try:
		name = plan9.read(cfd, 256)
		name = '%d' % (int(name))
		p = clone.rfind('/')
		dir = clone[:p]
		data = dir+'/'+name+'/data'
		cmd = 'connect '+dest
		if ds.local:
			cmd += ' '+ds.local
		plan9.write(cfd, cmd)
		fd = plan9.open(data, plan9.ORDWR)
		return (fd, dir+'/'+name, cfd)
	except:
		plan9.close(cfd)
		raise

def announce(addr):
	netdir, naddr = _nettrans(addr)

	cp = netdir.rfind('/')
	if cp == -1:
		raise Exception('announce arg format '+netdir)

	try:
		ctl = plan9.open(netdir, plan9.ORDWR)
	except Exception, e:
		raise Exception('announce opening '+netdir+': '+str(e))

	try:
		netdir = netdir[:cp]
		try:
			dir = netdir+'/'+plan9.read(ctl, 256)
		except Exception, e:
			raise Exception('announce reading '+netdir+': '+str(e))
		try:
			plan9.write(ctl, 'announce '+naddr)
		except Exception, e:
			raise Exception('announce writing '+netdir+': '+str(e))
	except:
		plan9.close(ctl)
		raise

	return ctl, dir

def listen(dir):
	buf = dir+'/listen'
	try:
		ctl = plan9.open(buf, plan9.ORDWR)
	except Exception, e:
		raise Exception('listen opening '+buf+': '+str(e))

	try:
		cp = dir.rfind('/')
		if cp == -1:
			raise Exception('listen arg format '+dir)
		try:
			ndir = dir[:cp+1]+plan9.read(ctl, 256)
		except:
			raise Exception('listen reading '+dir+': '+str(e))
		
	except:
		plan9.close(ctl)
		raise

	return ctl, ndir

def accept(ctl, dir):
	p = dir.rfind('/')
	if p==-1:
		num = dir
	else:
		num = dir[p+1:]
	try:
		plan9.write(ctl, 'accept '+num)
	except:
		# ignore return value, network might not need accepts
		pass
	return plan9.open(dir+'/data', plan9.ORDWR)

def reject(ctl, dir, cause):
	p = dir.rfind('/')
	if p==-1:
		num = dir
	else:
		num = dir[p+1:]
	plan9.write(ctl, 'reject '+num+' '+cause)

def _identtrans(netdir, addr):
	p = addr.find('!')
	proto = None
	naddr = None
	if p==-1:
		proto = addr
	else:
		proto, naddr = addr[:p], addr[p+1:]
	return netdir+'/'+proto+'/clone', naddr

def _nettrans(addr):
	p = addr.find('!')
	if p == -1:
		raise Exception('bad dial string: '+addr)

	proto, addr = addr[:p], addr[p+1:]
	netdir = '/net'
	if proto[0] == '/':
		p = proto.rfind('/')
		if p==0:
			raise Exception('bad dial string: '+addr)
		netdir, proto = proto[:p], proto[p+1:]
	addr = proto+'!'+addr

	try:
		fd = plan9.open(netdir+'/cs', plan9.ORDWR)
	except:
		return _identtrans(netdir, addr)

	try:
		plan9.write(fd, addr)
		plan9.seek(fd, 0, 0)
		buf = plan9.read(fd, 256)
	finally:
		plan9.close(fd)

	if not buf:
		raise Exception('short read from '+netdir+'/cs')

	p = buf.find(' ')
	if p == -1:
		raise Exception('bad response from '+netdir+'/cs')

	return buf[:p], buf[p+1:]



